'=========================================================================
'   LCD31.BAS     11-02-92       LCD/CY325 QuickBasic Examples
'                 10-07-99
'=========================================================================
' These are example drivers for the CY325 on the CYB003 prototyping board
' and they assume the 711A LCD is used in both 6x8 and 8x8 modes.
' To serially read data from the CY325 on the CYB003, be sure to close
' the serial turnaround switch #9 on the address switch.

' Version 31, OCT 1999, uses DOS Timer based delays instead of FOR loops,
' so the delay values below are not used at all.

	' $INCLUDE: 'QB.BI'


' Faster computers may need to increase the delay loops in these routines.
' The default delays are for an 8086 machine: Long=6k, Medium=4k, Short=2k.
' A '486 will need Delays of Long=500k, Medium=100k, and Short=20k.
' The three delay loop subroutines are at the end of the program,
' or the delay values may be changed here:

	LongDelay = 6000
	MediumDelay = 4000
	ShortDelay = 2000
	GOSUB ChangeDelays        'this is a simple keyboard input routine to
				  'allow delay values to be changed on-screen
				  'as well as implementing CTS
	CLS
	LOCATE 10, 30: PRINT "CY325 Demo on LCD"'print message on computer


' Open COM1 and set up 9600 baud, no parity, 8 data bits,
' and 1 stop bit, and disable the unused protocol lines.
' If DSR is not detected then Basic will timeout...
' so suppress RS(4),CTS(5),DSR(6),and CD(8) to avoid this.
' The CY325 supports a CTS signal which, if implemented on the serial line,
' would eliminate many of the delay loops in the programs.  The CYB-003
' Rev B board provides CTS at the serial jack, while Rev A did not.
' We'll use IF statements in order to support either case of CTS.
' If FlagCTS=1 then CTS is used and the delay loop is skipped.

				'open COM1 for random I/O
IF FlagCTS = 0 THEN OPEN "COM1:9600,N,8,1,RS,CS0,DS0,CD0" FOR RANDOM AS 1
IF FlagCTS = 1 THEN OPEN "COM1:9600,N,8,1,RS,CS5000,DS0,CD0" FOR RANDOM AS 1'uses CTS
'                                              \__change this value if your
'                                               computer times out too soon.


' Notes on the CY327:
' There are some minor cosmetic differences when using the CY327 and
' Hitachi-based displays, since the CY327 does not support separate planes
' for Graphics and Text (noticeable in LCD5). Moreover, the CY327 does
' not support custom Font or Large character mode (voiding LCD2). In
' addition, the CY327 has a special Dimension command to define the size
' and duty cycle of LCD being used, which should be reissued after every
' reset command. The D-command has no effect on the CY325. The format of
' the Dimension command is "D xBytes,DutyCycle"
'                  x-pixels\8___/       \__ 128, or 64, or 32 etc

'=========================================================================
LCD1: '     04-17-88       LCD/CY325 box, text, and bargraph examples
'=========================================================================

				'Set CY325 to command mode and
				'send initialize command to CY325.
				'BASIC appends the terminating CR:

	PRINT #1, CHR$(3)       'set to command mode with Ctrl-C

	PRINT #1, "I 0"         'Reset CY325 and LCD, a time consuming task
	GOSUB DelayLong         'pause while CY325 resets and clears screen
				'which takes about a second and a half


'-------------------------------------------------------------------------

' Certain CY325 operations take a long time, and therefore delay loops are
' necessary in the Basic code.  Otherwise subsequent instructions may be
' missed by the CY325 while it is busy with the Display.
' The time-consuming tasks are Screen Clear and Box or Window fill
' (I commands), Box and Window drawing commands (B and W),
' Large Character display, Scrolling, and 4x4 Key Scan.
' The shorter delays are not needed if CTS is implemented.


	PRINT #1, CHR$(3)       'set to command mode again with Ctrl-C since
				'the reset command defaults to Display mode.

	PRINT #1, "I 3"         'Erase entire screen, a time consuming task
	GOSUB DelayLong         'pause while CY325 resets and clears screen.

	PRINT #1, CHR$(3)       'set to command mode with Ctrl-C.

	PRINT #1, "/W 3,6,37,6" 'set up borderless Text Window;
				'the window is ready for text,
				'but we'll write into it later.
	IF FlagCTS = 0 THEN GOSUB DelayShort

	PRINT #1, "B 10,60,230,5"'draw a Graphics Box x1,y1,x2,y2
	IF FlagCTS = 0 THEN GOSUB DelayShort
				'pause while CY325 draws the box.
				'if the appropriate action is not apparent
				'then the pauses may have to be increased.
				'this pause is skipped when CTS is used.

	PRINT #1, "B 15,55,225,20"'draw a Graphics Box x1,y1,x2,y2
	IF FlagCTS = 0 THEN GOSUB DelayShort
				'pause while CY325 draws the box.
	PRINT #1, "/I 2"        'Fill in, or blacken, the current box
	GOSUB DelayLong

	PRINT #1, "B 20,50,220,25"'draw a Graphics Box x1,y1,x2,y2
	IF FlagCTS = 0 THEN GOSUB DelayShort
				'pause while CY325 draws the box.
	PRINT #1, "I 2"         'erase, or white out, the current box
	GOSUB DelayMedium

	PRINT #1, "/B 25,48,215,27"'set up borderless Graphics Box
	IF FlagCTS = 0 THEN GOSUB DelayShort
				'pause while CY325 phantom draws the box

	PRINT #1, CHR$(4)       'set to Display mode with Ctrl-D
	PRINT #1, "This is a Label for the Display  "
				'now write into the text Window that was
				'most recently defined by W or /W.
				'This borderless Window was defined by /W
				'as ranging from column 3 to col 37
				'so we have 34 characters available

				'--other print formats----------------------

	GOSUB DelayLong

	Title$ = "This is a new Label for Display"
	PRINT #1, USING "\                                \"; Title$

				'-------------------------------------------

	GOSUB DelayLong

	format$ = STRING$(34, 32)'this is a string of 34 spaces (SP=32)
	LSET format$ = Title$   'left justify title in format field
	PRINT #1, format$

'---------------------------------------------------------------------------
				'Put a picture into the graphics Box
				'that was defined by the last B or /B
				'command received by the CY325

	PRINT #1, CHR$(3)       'set to command mode with Ctrl-C
	IF FlagCTS = 0 THEN GOSUB DelayShort

				'first we'll try a bargraph or histograph
				'with 10 bars and with vertical values
				'within the vertical limits set by the box,
				'Top of 48 and Bottom of 27, leaving a
				'vertical range of 21 pixels

	PRINT #1, "H 10,3,8,15,19,16,18,15,12,7,5"
	IF FlagCTS = 0 THEN GOSUB DelayMedium


				'now we'll re-label the graph
	PRINT #1, CHR$(4)       'set to Display mode with Ctrl-D

	PRINT #1, "  J  F  M  A  M  J  J  A  S  O net"

	'is the same as the following
	Title$ = "  J  F  M  A  M  J  J  A  S  O net"
	PRINT #1, USING "\                                \"; Title$

								   
	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD1: Box,Text,Bargraph.   <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF


CLS

'==========================================================================
LCD2: '     04-17-88       CY325/LCD   Large font examples
'==========================================================================


	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "I 0"         'reset CY325 and clear screen

	GOSUB DelayLong         'pause for a while because
				'clearing everything takes a long time

	PRINT #1, "     Loading Large Character Font"
				'display message to LCD screen

	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C

'---------------------------------------------------------------------------
' To print a large character display we must first define the 2x2 char font.
' There will be a noticeable pause in the display as these definitions are
' set up in the CY325 memory.  They are stored in the following order:

'  char  0       1       2       3      ...
'       0 1     4 5     8 9    12 13    ...
'       2 3     6 7    10 11   14 15    ...

				'note that hex numbers must not begin with
				'an alpha character.  Therefore FFh must be
				'written as 0FFh, whereas 3h is valid.

'Large character 0
	PRINT #1, "F 0,0,0,0,0,0,0,0,0"
	PRINT #1, "F 1,0,0,0,0,0,0,0,0"
	PRINT #1, "F 2,0,0,0,0,0,0,0,0"                      'space
	PRINT #1, "F 3,0,0,0,0,0,0,0,0"
	IF FlagCTS = 0 THEN GOSUB DelayShort

'Large character 1      the eight rows
		 '    1   2   3   4   5   6   7   8
	PRINT #1, "F 4,3Fh,3Fh,3Fh,3Ch,3Ch,3Ch,3Ch,3Fh"
	PRINT #1, "F 5,38h,3Eh,3Fh,0Fh,07h,07h,06h,3Ch"      'B
	PRINT #1, "F 6,3Fh,3Ch,3Ch,3Ch,3Ch,3Fh,3Fh,3Fh"
	PRINT #1, "F 7,3Ch,06h,07h,07h,0Fh,3Fh,3Eh,38h"
	IF FlagCTS = 0 THEN GOSUB DelayShort

'Large character 2
	PRINT #1, "F 8,0Fh,0Fh,0Fh,03h,03h,03h,3h,3h"
	PRINT #1, "F 9,3Ch,3Ch,3Ch,30h,30h,30h,30h,30h"
	PRINT #1, "F 10,03h,03h,03h,03h,03h,0Fh,0Fh,0Fh"     'I
	PRINT #1, "F 11,30h,30h,30h,30h,30h,3Ch,3Ch,3Ch"
	IF FlagCTS = 0 THEN GOSUB DelayShort

'Large character 3
	PRINT #1, "F 12,07h,1Fh,3Fh,3Eh,3Ch,3Ch,3Ch,3Ch"
	PRINT #1, "F 13,38h,3Eh,3Fh,07h,03h,0,0,0"
	PRINT #1, "F 14,3Ch,3Ch,3Ch,3Ch,3Eh,3Fh,1Fh,07h"     'G
	PRINT #1, "F 15,3Fh,3Fh,0Fh,0Fh,0Fh,3Fh,3Fh,3Bh"
	IF FlagCTS = 0 THEN GOSUB DelayShort


	PRINT #1, "L"           'set large character mode with L-command.
	IF FlagCTS = 0 THEN GOSUB DelayMedium

	PRINT #1, CHR$(4)       'set CY325 into display mode with Ctrl-D.
				'A semicolon after this statement would
				'have suppressed the linefeed that caused
				'a large-character linefeed on the display.
	IF FlagCTS = 0 THEN GOSUB DelayMedium

	PRINT #1, "ABC"         'the ASCII representations of three of
				'the first four characters   0 1 2 3
				'the screen will display        BIG

	IF FlagCTS = 0 THEN GOSUB DelayShort
				'2 to 3 characters per pause may be the
				'maximum transfer rate from fast machines,
				'but one character per transfer is best.
				'print again on line 2, suppressing LF
	FOR J = 1 TO 3
	    PRINT #1, "@";              ' Space
		GOSUB DelayShort
	    PRINT #1, "A";              ' B
		GOSUB DelayShort        'this delay is not required and
	    PRINT #1, "B";              ' I       'varying it creates an
		GOSUB DelayShort                  'interesting display
	    PRINT #1, "C";              ' G
		GOSUB DelayShort
	NEXT J
	PRINT #1, ""            'print CRLF


				'let's go back to normal font

	PRINT #1, CHR$(3);      'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "/L"          'go back to normal font ( /L = not Large )
	PRINT #1, "C 7,0"       'place cursor at row 7 column 0

	PRINT #1, CHR$(4);      'set CY325 into display mode with Ctrl-D

	PRINT #1, "NORMAL AGAIN"  'bottom line of text

	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD2: Large Fonts.         <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF

'==========================================================================
LCD3: '     04-17-88       CY325/LCD   Read info from LCD
'==========================================================================
' The first thing to do is set up dimension statements for Basic.
' The registers are dimensioned for read operations on the CY325 and
' Binary$ is dimensioned to convert register contents to 1's and 0's.

	DIM Register$(20): DIM Binary$(20)
	CLS                     'clear the IBM screen
'---------------------------------------------------------------------------

	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	LOCATE 1, 20: PRINT "Setting the CY325 to Command Mode "

	PRINT #1, "I 0"         'reset CY325
	LOCATE 1, 20: PRINT "Resetting the CY325, Clearing LCD "
	GOSUB DelayLong
				'let's have the LCD display something
				'during the query operation:
	PRINT #1, "Mode Register Contents will display on computer screen"
				'this text will wrap around on LCD

	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	LOCATE 1, 20: PRINT "Setting the CY325 to Command Mode "
	LOCATE 1, 20: PRINT "Reading CY325 Registers over Com1 "
	GOSUB ReadRegisters     'read the default register parameters

	COLOR 15
	K = 1                   'for register#1
	LOCATE K + 2, 10: PRINT "Contents of Register "; K;
	LOCATE K + 2, 35: PRINT " = "; Register$(K); "  =  "; Binary$(K)
	GOSUB DelayMedium       'pause to view
	COLOR 7

		'In reading the registers we find the default value for
		'mode Register#1 is 3Fh or 0011 1111.  We will change that
		'register by setting bit 7 to turn on full-size cursor.
		'the new value will be 1011 1111 or BFh, written as 0BFh.

	PRINT #1, "M 1,0BFh"    'Notice the new cursor on the LCD.
	GOSUB DelayLong       'pause to view

				'now let's check that to see if it was set
	LOCATE 1, 20: PRINT "Reading CY325 Registers again     "
	GOSUB ReadRegisters     'read the default register parameters

	COLOR 15                'highlight register 1
	K = 1                   'for register#1
	LOCATE K + 2, 10: PRINT "Contents of Register "; K;
	LOCATE K + 2, 35: PRINT " = "; Register$(K); "  =  "; Binary$(K)
	LOCATE 23, 20: PRINT "Returning Control to LCD"
	COLOR 7
								   
	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD3: Using Registers.     <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF

CLS

'==========================================================================
LCD4: '     05-03-88       CY325/LCD   Graphics Examples
'==========================================================================
	DIM Row$(40)            'define up to 40 rows of graphics
				'data available

	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "I 0"         'reset CY325  and clear LCD
	GOSUB DelayLong
	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "V 0"         'define entire screen as a viewport

	PRINT #1, CHR$(3)       'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "/I 6"        'set to 8-bit (8x8) character mode
	GOSUB DelayLong


'----------------------------------------------------------------------

' The Graphics or G-command allows a number of pixels to be set on one row.
' First we will define the data that is to be sent, and then later we'll
' send it.  The 711 LCD has 640 pixels across, equivalent to 80 bytes of
' data per line.  The CY325 can only accept about 10 bytes at a time, or
' a few more if the serial interface is not used.
'----------------------------------------------------------------------

	'this is a telephone icon in 8x8 font
	'that is, each byte is eight bits wide

	Row$(1) = "G 5,000H,000H,000H,000H,000H"
	Row$(2) = "G 5,000H,000H,000H,000H,000H"
	Row$(3) = "G 5,000H,000H,000H,000H,000H"
	Row$(4) = "G 5,000H,000H,000H,000H,000H"
	Row$(5) = "G 5,000H,000H,000H,000H,000H"
	Row$(6) = "G 5,000H,000H,000H,000H,000H"
	Row$(7) = "G 5,000H,000H,000H,000H,000H"
	Row$(8) = "G 5,000H,000H,000H,000H,000H"

	Row$(9) = "G 5,000H,01FH,0FFH,0F8H,000H"
	Row$(10) = "G 5,000H,07FH,0FFH,0FEH,000H"
	Row$(11) = "G 5,001H,0FFH,0FFH,0FFH,080H"
	Row$(12) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"
	Row$(13) = "G 5,007H,0FFH,0FFH,0FFH,0E0H"
	Row$(14) = "G 5,007H,0FFH,0FFH,0FFH,0F0H"
	Row$(15) = "G 5,00FH,0FFH,0FFH,0FFH,0F0H"
	Row$(16) = "G 5,01FH,0FFH,0FFH,0FFH,0F8H"

	Row$(17) = "G 5,03FH,0E7H,000H,0E7H,0FCH"
	Row$(18) = "G 5,03FH,0C7H,000H,0E3H,0FCH"
	Row$(19) = "G 5,07FH,0C7H,000H,0E3H,0FEH"
	Row$(20) = "G 5,07FH,0C7H,000H,0E3H,0FEH"
	Row$(21) = "G 5,0FFH,0C7H,0FFH,0E3H,0FFH"
	Row$(22) = "G 5,0FFH,0C7H,0FFH,0E3H,0FFH"
	Row$(23) = "G 5,0FFH,0CFH,0FFH,0F3H,0FFH"
	Row$(24) = "G 5,0FFH,0DFH,0FFH,0FBH,0FFH"

	Row$(25) = "G 5,000H,03CH,063H,01CH,000H"
	Row$(26) = "G 5,000H,07CH,063H,01EH,000H"
	Row$(27) = "G 5,000H,07CH,063H,01EH,000H"
	Row$(28) = "G 5,000H,0FFH,0FFH,0FFH,000H"
	Row$(29) = "G 5,000H,0FCH,063H,01FH,000H"
	Row$(30) = "G 5,000H,0FCH,063H,01FH,000H"
	Row$(31) = "G 5,000H,0FCH,063H,01FH,000H"
	Row$(32) = "G 5,000H,0FFH,0FFH,0FFH,000H"

	Row$(33) = "G 5,001H,0FCH,063H,01FH,080H"
	Row$(34) = "G 5,001H,0FCH,063H,01FH,080H"
	Row$(35) = "G 5,003H,0FCH,063H,01FH,0C0H"
	Row$(36) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"
	Row$(37) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"
	Row$(38) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"
	Row$(39) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"
	Row$(40) = "G 5,003H,0FFH,0FFH,0FFH,0C0H"

'----------------------------------------------------------------------

	StartLocation$ = "G 0,10,6" 'place the cursor in starting position
	PRINT #1, StartLocation$    'at PixelRow 10 and CharColumn  6


	FOR J = 1 TO 40
	    PRINT #1, Row$(J)   'draw telephone icon
	NEXT J

	GOSUB DelayLong         'to see display

'----------------------------------------------------------------------

' This next example of Graphics is drawn in 6x8 character mode.  These
' graphics use the entire 711A LCD screen (8x40 character).  Because
' each graphics pixel line is 40 characters (240 pixels) long, and the
' CY325 can buffer about 10 bytes of graphics at a time, the screen
' will be drawn as four panels of 10 bytes each.

	PRINT #1, "I 6"         'go back to 6-bit character (6x8) font
				'and clear the screen.
	GOSUB DelayLong         'delay while loading font.
				'The prior telephone icon will appear to
				'shrink away because of resetting the char
				'mode back from 8-bit to 6-bit characters
				'without first clearing the screen.

'----------------------------------------------------------------------
' There are four pixel files containing the four panels of graphics,
' so this example will read the data from an external file.
' Each of the four panels will have a starting location:

	DIM StartLocation$(4)         'dimension BASIC array for four
				      'variables as needed below

	StartLocation$(1) = "G 0,0, 0"'start at PixelRow 0, CharCol  0
	StartLocation$(2) = "G 0,0,10"'start at PixelRow 0, CharCol 10
	StartLocation$(3) = "G 0,0,20"'start at PixelRow 0, CharCol 20
	StartLocation$(4) = "G 0,0,30"'start at PixelRow 0, CharCol 30

    FOR F = 1 TO 4
    PRINT #1, StartLocation$(F) 'place cursor at start of panel location
    DataFile$ = "FARM" + RIGHT$(STR$(F), 1) + ".PXL"
				'this is the name of a pixel file
    OPEN "I", #3, DataFile$
	FOR K = 1 TO 64         'the number of vertical pixels
	    Row$ = "G 10"       'first part of graphics command
	    FOR J = 1 TO 10     'the number of data bytes in command string
		INPUT #3, DataByte$           'read each data byte from file
		Row$ = Row$ + "," + DataByte$ 'and append to command string
	    NEXT J
	    PRINT #1, Row$      'print graphics on LCD
	    PRINT Row$          'print hex data to computer terminal
	NEXT K
    CLOSE #3
    NEXT F
    CLS                         'clear computer screen

' Drawing the screen in four panels was the easiest way to handle the
' data files.  However, the four panel files could have been set up as
' four open files, and one line could have been read from each file. The
' first line of each file could have been displayed on the LCD, at the
' appropriate start location. This would have resulted in one entire LCD
' width of graphics, one pixel high, being displayed on the screen.
' Displaying each subsequent line, with appropriate start locations,
' would result in a top to bottom painting of the screen. Alternatively,
' one large file, such as the result of LCD Paint, could be loaded one
' horizontal line at a time, in much the same way as reading 4 open files
' at one time.  An example QuickBasic Loader routine is printed in the
' LCD Paint manual.
								   
	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD4: Graphics.            <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF

CLS

'==========================================================================
LCD5: '     06-11-88       CY325/LCD   Special Font Examples
'==========================================================================
' This Special Font example will display the building blocks stored as
' special font characters.  These characters can represent giant pixels,
' and by writing these characters to specific screen locations, other larger
' characters can be created, or limited graphics can be drawn.
' ----------------------------------------------------------------------
' Normally the "I 6" command would be used here to reset the 6-bit character
' mode, and clear the screen, but the mode was reset in the last example.
' So, we won't clear the screen yet.  The next font example will superimpose
' over the top of the last graphics.  Then we'll clear only the graphics
' and leave the font exposed.  We will make use of the Viewport command "V",
' knowing the default viewport "V 0" is set when the part is initialized.
' The viewport has not been changed in any other example, so it is still at
' the default "V 0", which is the largest viewport, the entire screen.  All
' the prior examples assumed viewport 0.


	PRINT #1, "I 4"         'to re-load special fonts in 6-bit characters
				'required only because 8x8, "/I 6", erased it.
	GOSUB DelayLong         'delay while reloading font

	PRINT #1, CHR$(4);      'set CY325 into display mode with Ctrl-D
				'the semicolon suppresses the linefeed
				'which would have caused the cursor to
				'drop down a line

	PRINT #1, CHR$(14);     'set CY325 to special graphics with Ctrl-N
				'these are the giant pixel building block
				'characters for creating giant fonts

	PRINT #1, " @ A B C D E F G H I J K L M N O P"
				'the ASCII representations of special set

	GOSUB DelayShort        'to see display
'----------------------------------------------------------------------

				'print a mosaic pattern

	PRINT #1, "@@G@K@@@@@G@K@@@@@G@K@@@@@G@K@@@@@G@K@@";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "@@@O@@@@@@@O@@@@@@@O@@@@@@@O@@@@@@@O@@@";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "@@M@N@@@@@M@N@@@@@M@N@@@@@M@N@@@@@M@N@@";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "FF@@@FFFFF@@@FFFFF@@@FFFFF@@@FFFFF@@@FF";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "@@G@K@@@@@G@K@@@@@G@K@@@@@G@K@@@@@G@K@@";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "@@@O@@@@@@@O@@@@@@@O@@@@@@@O@@@@@@@O@@@";
	IF FlagCTS = 0 THEN GOSUB DelayShort
	PRINT #1, "@@M@N@@@@@M@N@@@@@M@N@@@@@M@N@@@@@M@N@@"
	IF FlagCTS = 0 THEN GOSUB DelayShort
				'the carriage return is not sent until the
				'last character, allowing the screen wrap
				'to handle the linefeeds.

	PRINT #1, CHR$(15);     'set CY325 back to normal font with Ctrl-O
	PRINT #1, CHR$(3);      'Set CY325 to command Mode with Ctrl-C
	GOSUB DelayLong         'it takes awhile to clear screen
	PRINT #1, "I 2"         'clear the graphics from ViewPort
				'but leave the text.
				'"I 1" would clear text and leave graphics.
	GOSUB DelayLong         'it takes awhile to clear screen
	IF FlagCTS = 0 THEN GOSUB DelayLong  'pause to empty CY325 buffer

	PRINT #1, "C 2,13"      'Place cursor at Row 2, column 13
				'rows are 0-7, columns 0-39
	PRINT #1, CHR$(4);      'set CY325 into display mode with Ctrl-D
	PRINT #1, "DONE"
	GOSUB DelayLong         'to see message

	PRINT #1, CHR$(3);      'Set CY325 to command Mode with Ctrl-C
	PRINT #1, "I 1"         'clear text from viewport
	GOSUB DelayLong         'it takes awhile to clear screen
	IF FlagCTS = 0 THEN GOSUB DelayLong  'pause to empty CY325 buffer
								   
	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD5: Special Fonts.       <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF

'=========================================================================
LCD6: '      10-08-88        LCD/CY325    viewport setting examples
'=========================================================================
	' In this example we will display all 256 viewports to get a feel for
	' which ones are valid, and for their size and location.
	CLS
	vStart = 0: vEnd = 255  'there are 255 possible viewports

	'------------------------first set up the computer to take input
	LOCATE 5, 20:  PRINT "SAMPLE VIEWPORTS "
	LOCATE 8, 20:  PRINT "Start Value [   ]"
	LOCATE 9, 20:  PRINT "End Value   [   ]"

	LOCATE 8, 33:  PRINT USING "###"; vStart
	LOCATE 9, 33:  PRINT USING "###"; vEnd
	LOCATE 11, 20: PRINT "<F1> to pause  <F2> to End "

	LOCATE 8, 33:  INPUT "", x$
	IF x$ <> "" THEN vStart = VAL(x$)
	LOCATE 8, 33:  PRINT USING "###"; vStart
	LOCATE 9, 33:  INPUT "", x$
	IF x$ <> "" THEN vEnd = VAL(x$)
	LOCATE 9, 33:  PRINT USING "###"; vEnd

	ON KEY(1) GOSUB KeyPause
	ON KEY(2) GOSUB LCD7        ' move on to last routine
				    ' or ON KEY(2) GOSUB EndProgram
				    ' is another possible option
	KEY(1) ON
	KEY(2) ON

'------------------------then start the LCD demo
	LOCATE 14, 20: PRINT "Viewport  Hex  Dec  Class"     'to screen
    FOR J = vStart TO vEnd
	IF J < 256 THEN Class = 3  '4   'these are the four classes of
	IF J < 192 THEN Class = 2  '3   'viewports, indicating the number
	IF J < 128 THEN Class = 1  '2   'of horiz viewports per screen
	IF J < 64 THEN Class = 0   '1
	'               Class_/      \_number of horizontal viewports

	Viewport$ = "V" + STR$(J)       'this is a decimal number, although
					'substitution of hex numbers would
					'show viewport structure better, as
					'in Viewport$="V 0"+Hex$(J)+"h"

	PRINT #1, CHR$(3);              'set to command mode
	PRINT #1, Viewport$
	PRINT #1, CHR$(4);              'set to display mode
	PRINT #1, Viewport$;            'display viewport value on LCD
	LOCATE 15, 20: PRINT Viewport$;
	LOCATE 15, 30: PRINT HEX$(J); "h"    'to screen
	LOCATE 15, 35: PRINT USING "###"; J
	LOCATE 15, 42: PRINT Class
	GOSUB DelayMedium               'to see viewport
	PRINT #1, CHR$(3);              'set to command mode
	PRINT #1, "/" + Viewport$       'erase viewport
	PRINT #1, "I 1"                 'clear text
    NEXT J
    KEY(1) OFF
								   
	ON KEY(2) GOSUB EndProgram
	KEY(2) ON

LOCATE 24, 1: PRINT "End of LCD6: Viewports.           <F2> to quit"
				'this pause gives time to digest the
				'info before starting next sequence
LOCATE 25, 35: PRINT "<CR> to continue"
INPUT "", x$    'wait for any input to continue

	KEY(2) OFF

'=========================================================================
LCD7: '      06-11-88        LCD/CY325    scroll/wrap routines
'=========================================================================
' This routine allows swapping between scrolling and wrapping modes
' to give the user a feel for the difference between the two.
' Delays can be added to slow down the screen a little to make
' it easier to see the differences.  It should also be noted that
' scroll/wrap is normally not used in this dynamic situation and that
' without use of CTS, it is possible to send the mode change command
' while the CY325 is not listening - it which case it may be ignored,
' or only some portion of the command will be received and will be
' improperly interpreted.


	CLS                     'first set up the computer screen:

	LOCATE 10, 20: PRINT "Test CY325 Scroll/Wrap Functions"
	ScrollFlag = 1          'begin with mode set to scroll
	LOCATE 12, 20: PRINT "    Now SCROLLING characters    "

	IF FlagCTS = 1 THEN     'manual dynamic mode changes can only
				'be performed with CTS implemented
		LOCATE 14, 20: PRINT "<F1> to Wrap        <F2> to End"
		ON KEY(1) GOSUB SetScroll     'define F1 key
		KEY(1) ON
	END IF
		ON KEY(2) GOSUB EndProgram
		KEY(2) ON
'------------------------------------------------------------------------

' then set CY325 to command mode
' send initialize command to CY325, BASIC appends the terminating CR


	PRINT #1, CHR$(3)       'set to command mode with Ctrl-C

	PRINT #1, "I 0"         'Reset CY325 and LCD, a time consuming task
	GOSUB DelayLong         'pause while CY325 resets and clears screen
				'which takes about a second and a half

	PRINT #1, CHR$(3)       'set to command mode with Ctrl-C

	wrap$ = "M 0,06h"       'these are the mode register values to
	scroll$ = "M 0,07h"     'change for scroll or wrap
				'and perform the change below

	IF ScrollFlag = 1 THEN PRINT #1, scroll$ ELSE PRINT #1, wrap$

	PRINT #1, CHR$(4);      'set to Display mode with Ctrl-D

	K = 0

PrintString:                    'this label defines the start of a loop
	K = K + 1
	FOR a = 32 TO 160       'The full character set.
	PRINT #1, CHR$(a);
	'GOSUB DelayShort        'Time to see each character
	NEXT a

IF FlagCTS = 0 THEN             'if CTS is not used then the program will
	GOSUB DelayShort
	IF K = 5 THEN           'automatically change between scroll and
		K = 0           'wrap without allowing the user to do so.
		GOSUB SetScroll 'to change modes automatically
		END IF
	END IF
GOTO PrintString                'loop until ESC breaks out


'=========================================================================
'                           subroutines
'=========================================================================

SetScroll: '-----subroutine to change from scroll to wrap--------

	KEY(1) STOP: KEY(2) STOP 'turn off these keys temporarily so that
				 'mode setting will not be interrupted.
IF ScrollFlag = 1 THEN           'If scrolling then
	ScrollFlag = 0           'set to wrap.
	PRINT #1, CHR$(3)        'Set to command mode with Ctrl-C
	PRINT #1, wrap$
	PRINT #1, CHR$(4);       'Set to Display mode with Ctrl-D
	LOCATE 12, 20: PRINT "    Now WRAPPING  characters    "
	IF FlagCTS = 1 THEN KEY(1) ON: LOCATE 14, 20: PRINT "<F1> to Scroll      <F2> to End"
	IF FlagCTS = 0 THEN LOCATE 14, 20: PRINT "          <F2> to End          "
	KEY(2) ON
	RETURN
	END IF

IF ScrollFlag = 0 THEN           'if wrapping then
	ScrollFlag = 1           'set to scroll.
	PRINT #1, CHR$(3)        'Set to command mode with Ctrl-C
	PRINT #1, scroll$
	PRINT #1, CHR$(4);       'set to Display mode with Ctrl-D
	LOCATE 12, 20: PRINT "    Now SCROLLING characters    "
	IF FlagCTS = 1 THEN KEY(1) ON: LOCATE 14, 20: PRINT "<F1> to Wrap        <F2> to End"
	IF FlagCTS = 0 THEN LOCATE 14, 20: PRINT "          <F2> to End          "
	KEY(2) ON
	RETURN
	END IF
	RETURN

KeyPause: '----------subroutine to allow a puase in screen write-----------
	LOCATE 12, 20: PRINT "<CR> to continue"
	INPUT "", x$    'wait for any input to continue
	LOCATE 12, 20: PRINT "                "
	RETURN

ReadRegisters: '-----subroutine to read register contents------------------
	'now get ready to read CY325 registers over COM1

	FOR K = 0 TO 19         'There are 0-19 registers.
	PRINT #1, "?"; K        'The command to query registers.
	INPUT #1, Register$(K)  'Read back from the CY325 over COM1.
				'Don't need to add a delay here because
				'the subroutine takes up a bit of time.
	GOSUB ConvertBinary     'We read a "?" and a Hex number, so convert
				'to binary representation and print to screen.
	LOCATE K + 2, 10: PRINT "Contents of Register "; K;
	LOCATE K + 2, 35: PRINT " = "; Register$(K); "  =  "; Binary$(K)
	NEXT K

	RETURN

ConvertBinary: '------subroutine to convert hex to binary----------------
	Binary$(K) = ""
			'Register$(K) from the CY325 will consist
			'of four characters:  a "?" with a 2-digit
			'hex number and an "h" suffix.
			'Ignore '?' and 'h' and convert 2-digit to binary
	Sample$ = MID$(Register$(K), 2, 1)'high byte
	GOSUB ConvertSample
	Binary$(K) = Binary$(K) + " "     'to separate high and low bytes
	Sample$ = MID$(Register$(K), 3, 1)'low byte
	GOSUB ConvertSample
	RETURN

ConvertSample: '------subroutine------------------

		Sample = VAL(Sample$)
		IF Sample$ = "F" THEN Sample = 15
		IF Sample$ = "E" THEN Sample = 14
		IF Sample$ = "D" THEN Sample = 13
		IF Sample$ = "C" THEN Sample = 12
		IF Sample$ = "B" THEN Sample = 11
		IF Sample$ = "A" THEN Sample = 10

		IF Sample - 8 >= 0 THEN
			GOSUB AddOne
			Sample = Sample - 8
		ELSE GOSUB AddZero
		END IF

		IF Sample - 4 >= 0 THEN
			GOSUB AddOne
			Sample = Sample - 4
		ELSE GOSUB AddZero
		END IF

		IF Sample - 2 >= 0 THEN
			GOSUB AddOne
			Sample = Sample - 2
		ELSE GOSUB AddZero
		END IF

		IF Sample - 1 >= 0 THEN GOSUB AddOne ELSE GOSUB AddZero
		RETURN

AddZero: '---subroutine------------------------
	Binary$(K) = Binary$(K) + "0"
	RETURN

AddOne: '---Subroutine------------------------
	Binary$(K) = Binary$(K) + "1"
	RETURN

'---------------------------------------------------------------------------

DelayLong: '-----time delay subroutine---------
	' 2.00 sec
	HSecDelay = 200         ' 2.00 sec
	GOSUB DOSTimeDelay
	' FOR I = 1 TO LongDelay
	' NEXT I
	RETURN

DelayMedium: '-----time delay subroutine---------
	' 1.20 sec
	HSecDelay = 120         ' 1.20 sec
	GOSUB DOSTimeDelay
	' FOR I = 1 TO MediumDelay
	' NEXT I
	RETURN

DelayShort: '-----time delay subroutine---------
	' 0.60 sec
	HSecDelay = 60          ' 0.60 sec
	GOSUB DOSTimeDelay
	' FOR I = 1 TO ShortDelay
	' NEXT I
	RETURN

DOSTimeDelay:

' This routine uses the Interrupt routine to call DOS for the current
' time, using DOS function 2Ch.  The delay argument is in hundredths
' of a second.
 
' Note: The Interrupt routine requires the Quick library QB.QLB(?). To
' load the library, start Quick Basic 4 with the /L switch:
'    qb4 /L
 
' The following include file QB.BI must also be present. It defines the
' RegType structure and the INTERRUPT function.
 
	DIM regs AS RegType            ' Define registers

' Get current DOS Time; set up input and do system call
	regs.ax = &H2C00
	CALL INTERRUPT(&H21, regs, regs)

' Time is returned in CH, CL, DH, DL as hours, min, sec, hund

' Extract information from registers
	DosHrs = (regs.cx AND &HFF00) / 256     ' isolate CH (0-23)
	DosMin = (regs.cx AND &HFF)             ' isolate CL (0-59)
	DosSec = (regs.dx AND &HFF00) / 256     ' isolate DH (0-59)
	DosHnd = (regs.dx AND &HFF)             ' isolate DL (0-99)

' compute the target time at the end of the delay
'  add 55 msec (6 hundredths) for system base tick resolution

	DosHnd = DosHnd + HSecDelay + 6
	OvrFlw = FIX(DosHnd / 100)
	DosHnd = DosHnd MOD 100

	DosSec = DosSec + OvrFlw
	OvrFlw = FIX(DosSec / 60)
	DosSec = DosSec MOD 60

	DosMin = DosMin + OvrFlw
	OvrFlw = FIX(DosMin / 60)
	DosMin = DosMin MOD 60

	DosHrs = DosHrs + OvrFlw                ' allow 24 incase of overflow!

DosTimeLoop:
	' IF INKEY$ = CHR$(27) THEN
	'       BEEP: CY545Flag = 0: Problem = 1
	'       CmdMsg$ = "Keyboard Interrupt": GOSUB PrintCmdMsg
	'       Msg$ = "Reset hardware/software": GOSUB PrintMsg
	'       RETURN 'CommandEntry
	' END IF

' Check current DOS Time; set up input and do system call
	regs.ax = &H2C00
	CALL INTERRUPT(&H21, regs, regs)

	ChkHrs = (regs.cx AND &HFF00) / 256     ' isolate CH (0-23)
	ChkMin = (regs.cx AND &HFF)             ' isolate CL (0-59)
	ChkSec = (regs.dx AND &HFF00) / 256     ' isolate DH (0-59)
	ChkHnd = (regs.dx AND &HFF)             ' isolate DL (0-99)

	IF (ChkHrs = 0) AND (DosHrs >= 23) THEN ChkHrs = 24

	IF ChkHrs < DosHrs THEN GOTO DosTimeLoop
	IF (ChkHrs = DosHrs) AND (ChkMin < DosMin) THEN GOTO DosTimeLoop
	IF (ChkMin = DosMin) AND (ChkSec < DosSec) THEN GOTO DosTimeLoop
	IF (ChkSec = DosSec) AND (ChkHnd < DosHnd) THEN GOTO DosTimeLoop

	RETURN


ChangeDelays: '------soubroutine to change delay values----------------------
	CLS
	LOCATE 1, 20: PRINT "    CY325 Demo on LCD      "

	' LOCATE 3, 15: PRINT "   Increase Delay Values for Faster Computers     "
	' LOCATE 5, 15: PRINT "8086 Long delay value   [      ]     (500k for a 486)"
	' LOCATE 5, 40: PRINT USING "######"; LongDelay

	' LOCATE 6, 15: PRINT "8086 Medium delay value [      ]     (100k for a 486)"
	' LOCATE 6, 40: PRINT USING "######"; MediumDelay

	' LOCATE 7, 15: PRINT "8086 Short delay value  [      ]     ( 20k for a 486)"
	' LOCATE 7, 40: PRINT USING "######"; ShortDelay

	' CTS line number was 8 with delays
	LOCATE 5, 15: PRINT "Is the CY325 CTS used?       [ ]"
	LOCATE 5, 45: PRINT ""; "Y"

	' LOCATE 5, 40: INPUT "", x$
	' IF x$ <> "" THEN LongDelay = VAL(x$)
	' LOCATE 5, 40: PRINT USING "######"; LongDelay

	' LOCATE 6, 40: INPUT "", x$
	' IF x$ <> "" THEN MediumDelay = VAL(x$)
	' LOCATE 6, 40: PRINT USING "######"; MediumDelay

	' LOCATE 7, 40: INPUT "", x$
	' IF x$ <> "" THEN ShortDelay = VAL(x$)
	' LOCATE 7, 40: PRINT USING "######"; ShortDelay

	LOCATE 5, 45: INPUT "", x$
	IF x$ = "N" OR x$ = "n" THEN FlagCTS = 0 ELSE FlagCTS = 1
	IF FlagCTS = 0 THEN x$ = "N"
	IF FlagCTS = 1 THEN x$ = "Y"
	LOCATE 5, 45: PRINT ""; x$

	RETURN

EndProgram: '----------------exit and return to DOS------------------------
	'When we exit, we will leave a message on LCD and on Computer.

	IF FlagCTS = 0 THEN GOSUB DelayLong  'pause to clear out any data
					     'remaining in CY325 buffer

	PRINT #1, CHR$(3);      'set to command mode with Ctrl-C
	PRINT #1, "V 0"         'set full screen viewport
	PRINT #1, "/I 2"        'Fill in, or blacken, the current window
	GOSUB DelayLong         'time to fill entire screen
	IF FlagCTS = 0 THEN GOSUB DelayLong 'more time to fill entire screen
	IF FlagCTS = 0 THEN GOSUB DelayLong 'even more time to fill screen

	PRINT #1, "I 1"         'erase, or white out, text in current window
	PRINT #1, "V 97h"       'draw viewport in center of screen
				'as current window
	GOSUB DelayMedium
	PRINT #1, CHR$(3);      'set to command mode with Ctrl-C
	PRINT #1, "I 2"         'erase graphics in current box
	GOSUB DelayMedium       'pause while CY325 clears window

	PRINT #1, CHR$(4)       'set CY325 into display mode with Ctrl-D
	PRINT #1, "  THE END"
	PRINT #1, CHR$(3);      'set to command mode with Ctrl-C
	PRINT #1, "C 1,1"       'place cursor at row 7 column 0
	GOSUB DelayLong         'time to see message

	'-----------------------------------------------------------------
	CLS
	LOCATE 12, 30: PRINT " END OF LCD DEMO "
	CLOSE 1                 'close the COM port
	END
	SYSTEM                  'return to DOS

'------------------------------END----------------------------------------

